@@ -105,6 +105,8 @@ public:
         FILTER_SHAPE_SHARP = 2   /*!< Sharp: Transition band is TBD of width. */
     };
 
+    static const unsigned int DEFAULT_FFT_SIZE = 8192;
+
     receiver(const std::string input_device="",
              const std::string audio_device="",
              unsigned int decimation=1);
@@ -159,11 +161,11 @@ public:
     status      set_freq_corr(double ppm);
     float       get_signal_pwr() const;
     void        set_iq_fft_size(int newsize);
-    void        set_iq_fft_window(int window_type);
-    void        get_iq_fft_data(std::complex<float>* fftPoints,
-                                unsigned int &fftsize);
-    void        get_audio_fft_data(std::complex<float>* fftPoints,
-                                   unsigned int &fftsize);
+    unsigned int iq_fft_size(void) const;
+    void        set_iq_fft_window(int window_type, bool normalize_energy);
+    int         get_iq_fft_data(float* fftPoints);
+    int         get_audio_fft_data(float* fftPoints);
+    unsigned int audio_fft_size(void) const;
 
     /* Noise blanker */
     status      set_nb_on(int nbid, bool on);
@@ -269,6 +271,8 @@ private:
 
     gr::blocks::multiply_const_ff::sptr audio_gain0; /*!< Audio gain block. */
     gr::blocks::multiply_const_ff::sptr audio_gain1; /*!< Audio gain block. */
+    gr::blocks::multiply_const_ff::sptr wav_gain0; /*!< WAV file gain block. */
+    gr::blocks::multiply_const_ff::sptr wav_gain1; /*!< WAV file gain block. */
 
     gr::blocks::file_sink::sptr         iq_sink;     /*!< I/Q file sink. */
 
