@@ -50,6 +50,7 @@ RemoteControl::RemoteControl(QObject *parent) :
     squelch_level = -150.0;
     audio_gain = -6.0;
     audio_recorder_status = false;
+    iq_recorder_status = false;
     receiver_running = false;
     hamlib_compatible = false;
     is_audio_muted = false;
@@ -57,8 +58,6 @@ RemoteControl::RemoteControl(QObject *parent) :
     rc_port = DEFAULT_RC_PORT;
     rc_allowed_hosts.append(DEFAULT_RC_ALLOWED_HOSTS);
 
-    rc_socket = 0;
-
     connect(&rc_server, SIGNAL(newConnection()), this, SLOT(acceptConnection()));
 }
 
@@ -77,12 +76,13 @@ void RemoteControl::start_server()
 /*! \brief Stop the server. */
 void RemoteControl::stop_server()
 {
-    if (rc_socket != 0) {
-        rc_socket->close();
-        rc_socket->deleteLater();
-        rc_socket = 0;
-    }
+	for(auto sock : rc_sockets) {
+		sock->close();
+		sock->deleteLater();
+	}
 
+	rc_sockets.clear();
+	
     if (rc_server.isListening())
         rc_server.close();
 
@@ -163,30 +163,34 @@ void RemoteControl::setHosts(QStringList hosts)
  */
 void RemoteControl::acceptConnection()
 {
-    if (rc_socket)
-    {
-        rc_socket->close();
-        rc_socket->deleteLater();
-    }
-    rc_socket = rc_server.nextPendingConnection();
+    auto socket = rc_server.nextPendingConnection();
+	rc_sockets.insert(socket);
 
     // check if host is allowed
-    auto address = rc_socket->peerAddress();
+    auto address = socket->peerAddress();
 
     for (auto allowed_host : rc_allowed_hosts)
     {
         if (address.isEqual(QHostAddress(allowed_host)))
         {
-            connect(rc_socket, SIGNAL(readyRead()), this, SLOT(startRead()));
+			connect(socket, SIGNAL(disconnected()), this, SLOT(socketDisconnect()));
+            connect(socket, SIGNAL(readyRead()), this, SLOT(startRead()));
             return;
         }
     }
 
     std::cout << "*** Remote connection attempt from " << address.toString().toStdString()
               << " (not in allowed list)" << std::endl;
-    rc_socket->close();
-    rc_socket->deleteLater();
-    rc_socket = 0;
+    socket->close();
+    socket->deleteLater();
+}
+
+void RemoteControl::socketDisconnect()
+{
+	QTcpSocket *socket = (QTcpSocket*)sender();
+	rc_sockets.erase(socket);
+	socket->close();
+	socket->deleteLater();	
 }
 
 /*! \brief Start reading from the socket.
@@ -196,6 +200,7 @@ void RemoteControl::acceptConnection()
  */
 void RemoteControl::startRead()
 {
+	QTcpSocket *rc_socket = (QTcpSocket*)sender();
     while (rc_socket->canReadLine()) {
         char    buffer[1024] = {0};
         int     bytes_read;
@@ -258,9 +263,9 @@ void RemoteControl::startRead()
         else if (cmd == "q" || cmd == "Q")
         {
             // FIXME: for now we assume 'close' command
+            rc_sockets.erase(rc_socket);
             rc_socket->close();
             rc_socket->deleteLater();
-            rc_socket = 0;
             return;
         }
         else
@@ -376,6 +381,7 @@ void RemoteControl::setAudioMuted(bool muted)
 /*! \brief Start audio recorder (from mainwindow). */
 void RemoteControl::startAudioRecorder(QString unused)
 {
+    /* Check first if a demodulation mode is set. */
     if (rc_mode > 0)
         audio_recorder_status = true;
 }
@@ -386,6 +392,18 @@ void RemoteControl::stopAudioRecorder()
     audio_recorder_status = false;
 }
 
+/*! \brief Start IQ recorder (from another window). */
+void RemoteControl::startIqRecorder(QString unused1, QString unused2)
+{
+    iq_recorder_status = true;
+}
+
+/*! \brief Stop IQ recorder (from another window). */
+void RemoteControl::stopIqRecorder()
+{
+    iq_recorder_status = false;
+}
+
 /*! \brief Set receiver status (from mainwindow). */
 void RemoteControl::setReceiverStatus(bool enabled)
 {
@@ -761,9 +779,11 @@ QString RemoteControl::cmd_get_func(QStringList cmdlist)
     QString func = cmdlist.value(1, "");
 
     if (func == "?")
-        answer = QString("RECORD DSP RDS MUTE\n");
+        answer = QString("RECORD IQRECORD DSP RDS MUTE\n");
     else if (func.compare("RECORD", Qt::CaseInsensitive) == 0)
         answer = QString("%1\n").arg(audio_recorder_status);
+    else if (func.compare("IQRECORD", Qt::CaseInsensitive) == 0)
+        answer = QString("%1\n").arg(iq_recorder_status);
     else if (func.compare("DSP", Qt::CaseInsensitive) == 0)
         answer = QString("%1\n").arg(receiver_running);
     else if (func.compare("RDS", Qt::CaseInsensitive) == 0)
@@ -786,7 +806,7 @@ QString RemoteControl::cmd_set_func(QStringList cmdlist)
 
     if (func == "?")
     {
-        answer = QString("RECORD DSP RDS MUTE\n");
+        answer = QString("RECORD IQRECORD DSP RDS MUTE\n");
     }
     else if ((func.compare("RECORD", Qt::CaseInsensitive) == 0) && ok)
     {
@@ -804,6 +824,22 @@ QString RemoteControl::cmd_set_func(QStringList cmdlist)
                 emit stopAudioRecorderEvent();
         }
     }
+    else if ((func.compare("IQRECORD", Qt::CaseInsensitive) == 0) && ok)
+    {
+        if (!receiver_running)
+        {
+            answer = QString("RPRT 1\n");
+        }
+        else
+        {
+            answer = QString("RPRT 0\n");
+            iq_recorder_status = status;
+            if (status)
+                emit startIqRecorderEvent();
+            else
+                emit stopIqRecorderEvent();
+        }
+    }
     else if ((func.compare("DSP", Qt::CaseInsensitive) == 0) && ok)
     {
         if (status)
