@@ -22,13 +22,8 @@
  */
 #pragma once
 
-#if GNURADIO_VERSION < 0x030800
-#include <gnuradio/filter/freq_xlating_fir_filter_ccf.h>
-#else
-#include <gnuradio/filter/freq_xlating_fir_filter.h>
-#endif
-
 #include <gnuradio/blocks/rotator_cc.h>
+#include <gnuradio/filter/freq_xlating_fir_filter.h>
 #include <gnuradio/hier_block2.h>
 
 class downconverter_cc;
