@@ -36,7 +36,7 @@ Bookmarks* Bookmarks::m_pThis = 0;
 
 Bookmarks::Bookmarks()
 {
-     TagInfo tag(TagInfo::strUntagged);
+     TagInfo::sptr tag = TagInfo::make(TagInfo::strUntagged);
      m_TagList.append(tag);
 }
 
@@ -61,14 +61,12 @@ void Bookmarks::add(BookmarkInfo &info)
     m_BookmarkList.append(info);
     std::stable_sort(m_BookmarkList.begin(),m_BookmarkList.end());
     save();
-    emit( BookmarksChanged() );
 }
 
 void Bookmarks::remove(int index)
 {
     m_BookmarkList.removeAt(index);
     save();
-    emit BookmarksChanged();
 }
 
 bool Bookmarks::load()
@@ -96,8 +94,8 @@ bool Bookmarks::load()
             QStringList strings = line.split(";");
             if(strings.count() == 2)
             {
-                TagInfo &info = findOrAddTag(strings[0]);
-                info.color = QColor(strings[1].trimmed());
+                TagInfo::sptr info = findOrAddTag(strings[0]);
+                info->color = QColor(strings[1].trimmed());
             }
             else
             {
@@ -127,7 +125,7 @@ bool Bookmarks::load()
                 QStringList TagList = strTags.split(",");
                 for(int iTag=0; iTag<TagList.size(); ++iTag)
                 {
-                  info.tags.append(&findOrAddTag(TagList[iTag].trimmed()));
+                  info.tags.append(findOrAddTag(TagList[iTag].trimmed()));
                 }
 
                 m_BookmarkList.append(info);
@@ -158,21 +156,20 @@ bool Bookmarks::save()
         stream << QString("# Tag name").leftJustified(20) + "; " +
                   QString(" color") << '\n';
 
-        QSet<TagInfo*> usedTags;
+        QMap<QString, TagInfo::sptr> usedTags;
         for (int iBookmark = 0; iBookmark < m_BookmarkList.size(); iBookmark++)
         {
             BookmarkInfo& info = m_BookmarkList[iBookmark];
-            for(int iTag = 0; iTag < info.tags.size(); ++iTag)
+            for (QList<TagInfo::sptr>::iterator iTag = info.tags.begin(); iTag < info.tags.end(); ++iTag)
             {
-              TagInfo& tag = *info.tags[iTag];
-              usedTags.insert(&tag);
+              usedTags.insert((*iTag)->name, *iTag);
             }
         }
 
-        for (QSet<TagInfo*>::iterator i = usedTags.begin(); i != usedTags.end(); i++)
+        for (QMap<QString, TagInfo::sptr>::iterator i = usedTags.begin(); i != usedTags.end(); i++)
         {
-            TagInfo& info = **i;
-            stream << info.name.leftJustified(20) + "; " + info.color.name() << '\n';
+            TagInfo::sptr info = *i;
+            stream << info->name.leftJustified(20) + "; " + info->color.name() << '\n';
         }
 
         stream << '\n';
@@ -192,17 +189,18 @@ bool Bookmarks::save()
                     QString::number(info.bandwidth).rightJustified(10) + "; ";
             for(int iTag = 0; iTag<info.tags.size(); ++iTag)
             {
-                TagInfo& tag = *info.tags[iTag];
-                if(iTag!=0)
+                TagInfo::sptr tag = info.tags[iTag];
+                if (iTag!=0)
                 {
                     line.append(",");
                 }
-                line.append(tag.name);
+                line.append(tag->name);
             }
 
             stream << line << '\n';
         }
 
+        emit BookmarksChanged();
         file.close();
         return true;
     }
@@ -222,7 +220,7 @@ QList<BookmarkInfo> Bookmarks::getBookmarksInRange(qint64 low, qint64 high)
     while (lb != ub)
     {
         const BookmarkInfo& info = *lb;
-        //if(info.IsActive())
+        if(info.IsActive())
         {
           found.append(info);
         }
@@ -233,7 +231,7 @@ QList<BookmarkInfo> Bookmarks::getBookmarksInRange(qint64 low, qint64 high)
 
 }
 
-TagInfo &Bookmarks::findOrAddTag(QString tagName)
+TagInfo::sptr Bookmarks::findOrAddTag(QString tagName)
 {
     tagName = tagName.trimmed();
 
@@ -245,8 +243,7 @@ TagInfo &Bookmarks::findOrAddTag(QString tagName)
     if (idx != -1)
         return m_TagList[idx];
 
-    TagInfo info;
-    info.name=tagName;
+    TagInfo::sptr info = TagInfo::make(tagName);
     m_TagList.append(info);
     emit TagListChanged();
     return m_TagList.last();
@@ -265,17 +262,17 @@ bool Bookmarks::removeTag(QString tagName)
         return false;
 
     // Delete Tag from all Bookmarks that use it.
-    TagInfo* pTagToDelete = &m_TagList[idx];
-    for(int i=0; i<m_BookmarkList.size(); ++i)
+    TagInfo::sptr pTagToDelete = m_TagList[idx];
+    for(int i=0; i < m_BookmarkList.size(); ++i)
     {
         BookmarkInfo& bmi = m_BookmarkList[i];
         for(int t=0; t<bmi.tags.size(); ++t)
         {
-            TagInfo* pTag = bmi.tags[t];
-            if(pTag == pTagToDelete)
+            TagInfo::sptr pTag = bmi.tags[t];
+            if(pTag.get() == pTagToDelete.get())
             {
                 if(bmi.tags.size()>1) bmi.tags.removeAt(t);
-                else bmi.tags[0] = &findOrAddTag(TagInfo::strUntagged);
+                else bmi.tags[0] = findOrAddTag(TagInfo::strUntagged);
             }
         }
     }
@@ -293,7 +290,7 @@ bool Bookmarks::setTagChecked(QString tagName, bool bChecked)
 {
     int idx = getTagIndex(tagName);
     if (idx == -1) return false;
-    m_TagList[idx].active = bChecked;
+    m_TagList[idx]->active = bChecked;
     emit BookmarksChanged();
     emit TagListChanged();
     return true;
@@ -304,7 +301,7 @@ int Bookmarks::getTagIndex(QString tagName)
     tagName = tagName.trimmed();
     for (int i = 0; i < m_TagList.size(); i++)
     {
-        if (m_TagList[i].name == tagName)
+        if (m_TagList[i]->name == tagName)
             return i;
     }
 
@@ -315,10 +312,10 @@ const QColor BookmarkInfo::GetColor() const
 {
     for(int iTag=0; iTag<tags.size(); ++iTag)
     {
-        TagInfo& tag = *tags[iTag];
-        if(tag.active)
+        TagInfo::sptr tag = tags[iTag];
+        if(tag->active)
         {
-            return tag.color;
+            return tag->color;
         }
     }
     return TagInfo::DefaultColor;
@@ -329,8 +326,8 @@ bool BookmarkInfo::IsActive() const
     bool bActive = false;
     for(int iTag=0; iTag<tags.size(); ++iTag)
     {
-        TagInfo& tag = *tags[iTag];
-        if(tag.active)
+        TagInfo::sptr tag = tags[iTag];
+        if(tag->active)
         {
             bActive = true;
             break;
