@@ -34,7 +34,7 @@ public:
     explicit BookmarksTagList(QWidget *parent = 0, bool bShowUntagged = true);
     QStringList getSelectedTags();
     void setSelectedTagsAsString(const QString& strTags);
-    void setSelectedTags(QList<TagInfo*> tags);
+    void setSelectedTags(QList<TagInfo::sptr>& tags);
     bool m_bUpdating;
 
 private:
